+++
title = "Properties"
weight = 2
chapter = true
+++

## Introduction

Apache ShardingSphere provides the way of property configuration to configure system level configuration.

## Configuration Item Explanation

| *Name*                              | *Data Type* | *Description*                                                                                                                                                                                                                                                                                                             | *Default Value* | *Dynamic Update* | 
| ----------------------------------- | ----------- |---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------| --------------- |----------------- | 
| sql-show (?)                        | boolean     | Whether show SQL or not in log. <br /> Print SQL details can help developers debug easier. The log details include: logic SQL, actual SQL and SQL parse result. <br /> Enable this property will log into log topic `ShardingSphere-SQL`, log level is INFO.                                                              | false           | true             |
| sql-simple (?)                      | boolean     | Whether show SQL details in simple style.                                                                                                                                                                                                                                                                                 | false           | true             |
| kernel-executor-size (?)            | int         | The max thread size of worker group to execute SQL. One ShardingSphereDataSource will use a independent thread pool, it does not share thread pool even different data source in same JVM.                                                                                                                                | infinite        | false            |
| max-connections-size-per-query (?)  | int         | Max opened connection size for each query.                                                                                                                                                                                                                                                                                | 1               | true             |
| check-table-metadata-enabled (?)    | boolean     | Whether validate table meta data consistency when application startup or updated.                                                                                                                                                                                                                                         | false           | false            |
| proxy-frontend-flush-threshold (?)  | int         | Flush threshold for every records from databases for ShardingSphere-Proxy.                                                                                                                                                                                                                                                | 128             | true             |
| proxy-opentracing-enabled (?)       | boolean     | Whether enable opentracing for ShardingSphere-Proxy.                                                                                                                                                                                                                                                                      | false           | true             |
| proxy-hint-enabled (?)              | boolean     | Whether enable hint for ShardingSphere-Proxy. Using Hint will switch proxy thread mode from IO multiplexing to per connection per thread, which will reduce system throughput.                                                                                                                                            | false           | true             |
| proxy-backend-query-fetch-size (?)  | int         | Proxy backend query fetch size. A larger value may increase the memory usage of ShardingSphere Proxy. The default value is -1, which means set the minimum value for different JDBC drivers.                                                                                                                              | -1              | false            |
| check-duplicate-table-enabled (?)   | boolean     | Whether validate duplicate table when application startup or updated.                                                                                                                                                                                                                                                     | false           | false            |
| proxy-frontend-executor-size (?)    | int         | Proxy frontend Netty executor size. The default value is 0, which means let Netty decide.                                                                                                                                                                                                                                 | 0               | false            |
| proxy-backend-executor-suitable (?) | String      | Available options of proxy backend executor suitable: OLAP(default), OLTP. The OLTP option may reduce time cost of writing packets to client, but it may increase the latency of SQL execution and block other clients if client connections are more than `proxy-frontend-executor-size`, especially executing slow SQL. | OLAP            | false            |
| proxy-frontend-max-connections (?)  | int         | The maximum permitted number of client connections to Proxy. The default value is 0 and less than or equal to 0 means no limitation.                                                                                                                                                                                      | 0               | true             |
| sql-federation-enabled (?)          | boolean     | Whether enable sql federation.                                                                                                                                                                                                                                                                                            | false           | true             |
| show-process-list-enabled (?)       | boolean     | Whether enable show process list, and it only take effect when mode is Cluster. This function is similar as MySQL show processlist. It just apply on DDL and DML statements currently.                                                                                                                                    | false           | true             |

Properties can be updated by [DistSQL#RAL](/en/user-manual/shardingsphere-proxy/distsql/syntax/ral/).
Dynamic update can take effect immediately, static update can take effect after restarted.
